/**
 *  LibVortex:  A BEEP (RFC3080/RFC3081) implementation.
 *  Copyright (C) 2005 Advanced Software Production Line, S.L.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public License
 *  as published by the Free Software Foundation; either version 2.1 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307 USA
 *  
 *  You may find a copy of the license under this software is released
 *  at COPYING file. This is LGPL software: you are welcome to
 *  develop proprietary applications using this library without any
 *  royalty or fee but returning back any change, improvement or
 *  addition in the form of source code, project image, documentation
 *  patches, etc. 
 *
 *  For commercial support on build BEEP enabled solutions contact us:
 *          
 *      Postal address:
 *         Advanced Software Production Line, S.L.
 *         C/ Dr. Michavila N 14
 *         Coslada 28820 Madrid
 *         Spain
 *
 *      Email address:
 *         info@aspl.es - http://fact.aspl.es
 */
#ifndef __VORTEX_H__
#define __VORTEX_H__

/**
 * \addtogroup vortex
 * @{
 */

/* External header includes */
#include <glib.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

/* Axl library headers */
#include <axl.h>

/* Direct portable mapping definitions */
#ifdef G_OS_UNIX

/* Portable socket definition while using Vortex Library */
#define VORTEX_SOCKET          gint
#define VORTEX_INVALID_SOCKET  -1
#define VORTEX_SOCKET_ERROR    -1
#define vortex_close_socket    close
#define vortex_getpid          getpid
#define vortex_sscanf          sscanf
#define vortex_is_disconnected (errno == EPIPE)
#endif
#ifdef G_OS_WIN32
#include <winsock2.h>
#include <windows.h>
#include <fcntl.h>
#include <io.h>
#include <process.h>
#define VORTEX_SOCKET          SOCKET
#define VORTEX_INVALID_SOCKET  INVALID_SOCKET
#define VORTEX_SOCKET_ERROR    SOCKET_ERROR
#define vortex_close_socket    closesocket
#define vortex_getpid          _getpid
#define vortex_sscanf          sscanf
#define uint16_t               u_short
#define vortex_is_disconnected ((errno == WSAESHUTDOWN) || (errno == WSAECONNABORTED) || (errno == WSAECONNRESET))


/* Additional header definition to map vortex library into windows */
#ifdef  EINTR
#undef  EINTR
#endif
#define EINTR WSAEINTR
#define EWOULDBLOCK WSAEWOULDBLOCK
#define SHUT_RDWR SD_BOTH
#endif



BEGIN_C_DECLS

/* Internal includes and external includes for Vortex API
 * consumers. */
#include <vortex_types.h>
#include <vortex_xml_rpc_types.h>
#include <vortex_support.h>
#include <vortex_thread_pool.h>
#include <vortex_semaphore.h>
#include <vortex_queue.h>
#include <vortex_hash.h>
#include <vortex_handlers.h>
#include <vortex_connection.h>
#include <vortex_listener.h>
#include <vortex_frame_factory.h>
#include <vortex_greetings.h>
#include <vortex_profiles.h>
#include <vortex_channel.h>
#include <vortex_channel_pool.h>
#include <vortex_io.h>
#include <vortex_reader.h>
#include <vortex_dtds.h>
#include <vortex_sequencer.h>
#include <vortex_writer.h>
#include <vortex_channel_pool.h>
#include <vortex_errno.h>
#include <vortex_tls.h>
#include <vortex_sasl.h>
#include <vortex_xml_rpc.h>

END_C_DECLS

#ifdef G_OS_UNIX
#include <fcntl.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/select.h>
#include <sys/time.h>
#include <unistd.h>
#endif


#ifdef G_OS_WIN32
#include <vortex_win32.h>
#else
#include <errno.h>
#endif

/* console debug support:
 *
 * If enabled, the log reporting is activated as usual. If log is
 * stripped from vortex building all instructions are removed.
 */
#ifdef ENABLE_VORTEX_LOG
#define vortex_log(domain, level, message, ...)  g_log(domain, level, message, ##__VA_ARGS__)
#else
#define vortex_log(domain, level, message, ...) /* nothing */
#endif

gboolean vortex_init                 ();

void     vortex_exit                 ();

gboolean vortex_log_is_enabled       ();

void     vortex_log_enable           (gboolean status);

gboolean vortex_color_log_is_enabled ();

void     vortex_color_log_enable     (gboolean status);

/* @} */
#endif
