/**
 *  LibVortex:  A BEEP (RFC3080/RFC3081) implementation.
 *  Copyright (C) 2005 Advanced Software Production Line, S.L.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public License
 *  as published by the Free Software Foundation; either version 2.1 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307 USA
 *  
 *  You may find a copy of the license under this software is released
 *  at COPYING file. This is LGPL software: you are welcome to
 *  develop proprietary applications using this library without any
 *  royalty or fee but returning back any change, improvement or
 *  addition in the form of source code, project image, documentation
 *  patches, etc. 
 *
 *  For commercial support on build BEEP enabled solutions contact us:
 *          
 *      Postal address:
 *         Advanced Software Production Line, S.L.
 *         C/ Dr. Michavila N 14
 *         Coslada 28820 Madrid
 *         Spain
 *
 *      Email address:
 *         info@aspl.es - http://fact.aspl.es
 */
#ifndef __VORTEX_CONNECTION_H__
#define __VORTEX_CONNECTION_H__

#include <vortex.h>


VortexConnection  * vortex_connection_new                    (gchar * host, gchar * port,
							      VortexConnectionNew on_connected, 
							      gpointer user_data);

gboolean            vortex_connection_reconnect              (VortexConnection * connection,
							      VortexConnectionNew on_connected,
							      gpointer user_data);

gboolean            vortex_connection_close                  (VortexConnection * connection);

gboolean            vortex_connection_close_all_channels     (VortexConnection * connection, 
							      gboolean also_channel_0);

void                vortex_connection_ref                    (VortexConnection * connection,
							      gchar            * who);

void                vortex_connection_unref                  (VortexConnection * connection,
							      gchar            * who);

gint                vortex_connection_ref_count              (VortexConnection * connection);

VortexConnection  * vortex_connection_new_empty              (gint                session,
							      VortexPeerRole      role);

VortexConnection  * vortex_connection_new_empty_from_connection (VORTEX_SOCKET socket, 
								 VortexConnection * __connection,
								 VortexPeerRole role);

void                vortex_connection_timeout                (glong miliseconds_to_wait);

glong               vortex_connection_get_timeout            ();

gboolean            vortex_connection_is_ok                  (VortexConnection * connection, 
							      gboolean free_on_fail);

gchar             * vortex_connection_get_message            (VortexConnection * connection);

void                vortex_connection_free                   (VortexConnection * connection);

GList             * vortex_connection_get_remote_profiles    (VortexConnection * connection);

void                vortex_connection_set_remote_profiles    (VortexConnection * connection,
							      GList            * profiles);

gboolean            vortex_connection_is_profile_supported   (VortexConnection * connection, 
							      gchar * uri);

gboolean            vortex_connection_channel_exists         (VortexConnection * connection, 
							      gint channel_num);

void                vortex_connection_foreach_channel        (VortexConnection * connection,
							      GHFunc func,
							      gpointer user_data);

gint                vortex_connection_get_next_channel       (VortexConnection * connection);

VortexChannel     * vortex_connection_get_channel            (VortexConnection * connection, 
							      gint channel_num);

VORTEX_SOCKET       vortex_connection_get_socket             (VortexConnection * connection);

void                vortex_connection_set_close_socket       (VortexConnection * connection,
							      gboolean action);

void                vortex_connection_add_channel            (VortexConnection * connection, 
							      VortexChannel * channel);

void                vortex_connection_remove_channel         (VortexConnection * connection, 
							      VortexChannel * channel);

gchar             * vortex_connection_get_host               (VortexConnection * connection);

gchar             * vortex_connection_get_port               (VortexConnection * connection);

gint                vortex_connection_get_id                 (VortexConnection * connection);

gboolean            vortex_connection_set_blocking_socket    (VortexConnection * connection);

gboolean            vortex_connection_set_nonblocking_socket (VortexConnection * connection);

void                vortex_connection_set_data               (VortexConnection * connection,
							      gchar            * key,
							      gpointer           value);

void                vortex_connection_set_data_full          (VortexConnection * connection,
							      gchar            * key,
							      gpointer           value,
							      GDestroyNotify     key_destroy,
							      GDestroyNotify     value_destroy);

void                vortex_connection_set_auto_tls           (gboolean           enabled,
							      gboolean           allow_tls_failures,
							      gchar            * serverName);

gpointer            vortex_connection_get_data               (VortexConnection * connection,
							      gchar            * key);
							      

VortexChannelPool * vortex_connection_get_channel_pool       (VortexConnection * connection,
							      gint               pool_id);

gint                vortex_connection_get_pending_msgs       (VortexConnection * connection);

VortexPeerRole      vortex_connection_get_role               (VortexConnection * connection);

gchar             * vortex_connection_get_features           (VortexConnection * connection);

gchar             * vortex_connection_get_localize           (VortexConnection * connection);

VortexSendHandler      vortex_connection_set_send_handler    (VortexConnection * connection,
							      VortexSendHandler  send_handler);

VortexReceiveHandler   vortex_connection_set_receive_handler (VortexConnection * connection,
							      VortexReceiveHandler receive_handler);

void                   vortex_connection_set_default_io_handler (VortexConnection * connection);
								 

VortexConnectionOnClose vortex_connection_set_on_close       (VortexConnection * connection,
							      VortexConnectionOnClose on_close_handler);

VortexConnectionOnCloseFull vortex_connection_set_on_close_full  (VortexConnection * connection,
								  VortexConnectionOnCloseFull on_close_handler,
								  gpointer data);

gpointer                   vortex_connection_remove_on_close_full (VortexConnection              * connection, 
								   VortexConnectionOnCloseFull     on_close_handler);

gint                vortex_connection_invoke_receive         (VortexConnection * connection,
							      gchar            * buffer,
							      gint               buffer_len);

gint                vortex_connection_invoke_send            (VortexConnection * connection,
							      gchar            * buffer,
							      gint               buffer_len);

void                vortex_connection_sanity_socket_check    (gboolean enable);

gboolean            vortex_connection_parse_greetings_and_enable (VortexConnection * connection, 
								  VortexFrame      * frame);

void                vortex_connection_set_preread_handler        (VortexConnection * connection, 
								  VortexConnectionOnPreRead pre_accept_handler);

gboolean            vortex_connection_is_defined_preread_handler (VortexConnection * connection);

void                vortex_connection_invoke_preread_handler     (VortexConnection * connection);

void                vortex_connection_set_tlsfication_status     (VortexConnection * connection,
								  gboolean           status);

gboolean            vortex_connection_is_tlsficated              (VortexConnection * connection);


/** 
 * @internal
 * Do not use the following functions, internal Vortex Library purposes.
 */
void                __vortex_connection_set_not_connected    (VortexConnection * connection, 
							      gchar            * message);

gint                vortex_connection_do_a_sending_round     (VortexConnection * connection);

void                vortex_connection_lock_channel_pool      (VortexConnection * connection);

void                vortex_connection_unlock_channel_pool    (VortexConnection * connection);

gint                vortex_connection_next_channel_pool_id   (VortexConnection * connection);

void                vortex_connection_add_channel_pool       (VortexConnection  * connection,
							      VortexChannelPool * pool);

gboolean            __vortex_connection_parse_greetings      (VortexConnection * connection, 
							      VortexFrame * frame);

/** 
 * @internal
 *
 * Set of functions to support storing a free handler per item not
 * only for the connection function but also for the channel
 * functions.
 */
gpointer             __vortex_connection_data_create (gpointer key,  GDestroyNotify key_destroy_func,
							gpointer data, GDestroyNotify data_destroy_func);

void                 __vortex_connection_data_free_value (gpointer data);

gpointer             __vortex_connection_data_get (gpointer data);
#endif
