/**
 *  LibVortex:  A BEEP (RFC3080/RFC3081) implementation.
 *  Copyright (C) 2005 Advanced Software Production Line, S.L.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public License
 *  as published by the Free Software Foundation; either version 2.1 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307 USA
 *  
 *  You may find a copy of the license under this software is released
 *  at COPYING file. This is LGPL software: you are welcome to
 *  develop proprietary applications using this library without any
 *  royalty or fee but returning back any change, improvement or
 *  addition in the form of source code, project image, documentation
 *  patches, etc. 
 *
 *  For commercial support on build BEEP enabled solutions contact us:
 *          
 *      Postal address:
 *         Advanced Software Production Line, S.L.
 *         C/ Dr. Michavila N 14
 *         Coslada 28820 Madrid
 *         Spain
 *
 *      Email address:
 *         info@aspl.es - http://fact.aspl.es
 */
#ifndef __VORTEX_FRAME_FACTORY_H__
#define __VORTEX_FRAME_FACTORY_H__

#include <vortex.h>

gchar *        vortex_frame_build_up_from_params (VortexFrameType   type,
						  gint32            channel,
						  gint32            msgno,
						  gboolean          more,
						  guint32           seqno,
						  gint32            size,
						  gint32            ansno,
						  gchar           * payload);

gchar *        vortex_frame_build_up_from_params_s (VortexFrameType   type,
						    gint32            channel,
						    gint32            msgno,
						    gboolean          more,
						    guint32           seqno,
						    gint32            size,
						    gint32            ansno,
						    gchar  *          content_type,
						    gchar  *          transfer_encoding,
						    gchar  *          payload,
						    gint   *          frame_size);

gchar *        vortex_frame_seq_build_up_from_params (gint channel_num,
						      gint ackno,
						      gint window_size);

VortexFrame * vortex_frame_create               (VortexFrameType   type,
						 gint32            channel,
						 gint32            msgno,
						 gboolean          more,
						 guint32           seqno,
						 gint32            size,
						 gint32            ansno,
						 gchar           * payload);

VortexFrame * vortex_frame_create_full          (VortexFrameType   type,
						 gint32            channel,
						 gint32            msgno,
						 gboolean          more,
						 guint32           seqno,
						 gint32            size,
						 gint32            ansno,
						 gchar           * content_type,
						 gchar           * transfer_encoding,
						 gchar           * payload);

VortexFrame * vortex_frame_copy                 (VortexFrame      * frame);

gchar *       vortex_frame_get_raw_frame         (VortexFrame * frame);

VortexFrame * vortex_frame_get_next              (VortexConnection * connection);

gboolean      vortex_frame_send_raw              (VortexConnection * connection, 
						  gchar * a_frame, 
						  gint frame_size);

void          vortex_frame_free                  (VortexFrame * frame);

VortexFrame * vortex_frame_join                  (VortexFrame * a, 
						  VortexFrame * b);

VortexFrame * vortex_frame_join_extending       (VortexFrame * a, 
						 VortexFrame * b);

gboolean      vortex_frame_are_joinable          (VortexFrame * a, 
						  VortexFrame * b);

gboolean      vortex_frame_are_equal             (VortexFrame * a, 
						  VortexFrame * b);

gint          vortex_frame_get_id                (VortexFrame * frame);

VortexFrameType vortex_frame_get_type            (VortexFrame * frame);

gchar       * vortex_frame_get_content_type      (VortexFrame * frame);

gchar       * vortex_frame_get_transfer_encoding (VortexFrame * frame);

gint          vortex_frame_get_mime_header_size  (VortexFrame * frame);

gint          vortex_frame_get_channel           (VortexFrame * frame);

gint          vortex_frame_get_msgno             (VortexFrame * frame);

gint          vortex_frame_get_more_flag         (VortexFrame * frame);

gint          vortex_frame_get_seqno             (VortexFrame * frame);

gint          vortex_frame_get_payload_size      (VortexFrame * frame);

gint          vortex_frame_get_content_size      (VortexFrame * frame);

gint          vortex_frame_get_ansno             (VortexFrame * frame);

gchar       * vortex_frame_get_payload           (VortexFrame * frame);

gchar       * vortex_frame_get_ok_message        ();

gchar       * vortex_frame_get_error_message     (gchar * code, 
						  gchar * error_content,
						  gchar * xml_lang);

gboolean      vortex_frame_is_error_message      (VortexFrame * frame,
						  gchar ** code,
						  gchar ** message);

gchar       * vortex_frame_get_start_message     (gint             channel_num,
						  gchar          * serverName, 
						  gchar          * profile,  
						  VortexEncoding   encoding,
						  gchar          * content_profile,
						  gint             profile_content_size);

gchar       * vortex_frame_get_start_rpy_message (gchar * profile, 
						  gchar * profile_content);

gchar       * vortex_frame_get_close_message     (gint    number,
						  gchar * code,
						  gchar * xml_lang,
						  gchar * close_content);
#endif
