/**
 *  LibVortex:  A BEEP (RFC3080/RFC3081) implementation.
 *  Copyright (C) 2005 Advanced Software Production Line, S.L.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public License
 *  as published by the Free Software Foundation; either version 2.1 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this program; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307 USA
 *  
 *  You may find a copy of the license under this software is released
 *  at COPYING file. This is LGPL software: you are welcome to
 *  develop proprietary applications using this library without any
 *  royalty or fee but returning back any change, improvement or
 *  addition in the form of source code, project image, documentation
 *  patches, etc. 
 *
 *  For commercial support on build BEEP enabled solutions contact us:
 *          
 *      Postal address:
 *         Advanced Software Production Line, S.L.
 *         C/ Dr. Michavila N 14
 *         Coslada 28820 Madrid
 *         Spain
 *
 *      Email address:
 *         info@aspl.es - http://fact.aspl.es
 */
#include <vortex.h>

#define LOG_DOMAIN "vortex-win32"

#ifdef G_OS_WIN32

void vortex_win32_init ()
{
	WORD wVersionRequested; 
	WSADATA wsaData; 
	int error; 
	
	wVersionRequested = MAKEWORD( 2, 2 ); 
	
	error = WSAStartup( wVersionRequested, &wsaData ); 
	if (error != NO_ERROR) {
		vortex_log (LOG_DOMAIN, G_LOG_LEVEL_CRITICAL, "unable to init winsock api, exiting..");
		
		exit (0);
	}
	vortex_log (LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "winsock initialization ok");
	return;
}

BOOL APIENTRY DllMain (HINSTANCE hInst,
                       DWORD reason,
                       LPVOID reserved)
{
 
    /* always returns TRUE because vortex init is done through
     * vortex_init */
    return TRUE;
}

gboolean __vortex_win32_blocking_socket_set (VORTEX_SOCKET socket,
                                             gint status) 
{
        unsigned long enable = status;

	return (ioctlsocket (socket, FIONBIO, &enable) == 0);
}

gboolean vortex_win32_nonblocking_enable (VORTEX_SOCKET socket)
{
        return __vortex_win32_blocking_socket_set (socket, 1);
}

gboolean vortex_win32_blocking_enable (VORTEX_SOCKET socket)
{
        return __vortex_win32_blocking_socket_set (socket, 0);
}

#endif
