//  LibVortex:  A BEEP implementation for af-arch
//  Copyright (C) 2005 Advanced Software Production Line, S.L.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or   
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of 
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <vortex.h>
#include <glib.h>


#define PLAIN_PROFILE "http://fact.aspl.es/profiles/plain_profile"

void frame_received (VortexChannel    * channel,
		     VortexConnection * connection,
		     VortexFrame      * frame,
		     gpointer           user_data)
{
	int iterator = 10;
	g_print ("A frame received on channl: %d\n",     vortex_channel_get_number (channel));
	g_print ("Data received: '%s'\n",                vortex_frame_get_payload (frame));

	// reply the peer client with the same content 10 times
	while (iterator >= 0) {
		g_print ("Sending the reply..\n");
		if (! vortex_channel_send_ans_rpyv (channel,
						    vortex_frame_get_msgno (frame),
						    "Received Ok(%d): %s",
						    iterator,
						    vortex_frame_get_payload (frame))) {
			g_critical ("There was an error while sending the reply message");
		}
		g_print ("Reply: %d sent..\n", iterator);
		iterator--;
	}

	// send the last reply.
	if (!vortex_channel_finalize_ans_rpy (channel, vortex_frame_get_msgno (frame))) {
		g_critical ("There was an error while sending the NUL reply message");
	}
				
	g_print ("VORTEX_LISTENER: end task (pid: %d)\n", getpid ());


	return;
}

gboolean start_channel (gint               channel_num, 
			VortexConnection * connection, 
			gpointer           user_data)
{
	// implement profile requirement for allowing starting a new channel
	// to return FALSE denies channel creation
	// to return TRUE allows create the channel
	return TRUE;
}

gboolean close_channel (gint               channel_num, 
			VortexConnection * connection, 
			gpointer           user_data)
{
	// implement profile requirement for allowing to closeing a the channel
	// to return FALSE denies channel closing
	// to return TRUE allows to close the channel
	return TRUE;
}

gint main (gint argc, gchar ** argv) 
{

	// init vortex library
	vortex_init ();

	// register a profile
	vortex_profiles_register (PLAIN_PROFILE, 
				  start_channel, NULL, 
				  close_channel, NULL,
				  frame_received, NULL);
	
	vortex_greetings_set_features ("enable-tls");
	vortex_greetings_set_localize ("es-ES");

	// create a vortex server
	vortex_listener_new ("0.0.0.0", "44000", NULL, NULL);

	// wait for listeners (until vortex_exit is called)
	vortex_listener_wait ();
	
	// end vortex function
	vortex_exit ();

	return 0;
}

