/* Hey emacs, show me this like 'c': -*- c -*-
 *
 * xml-rpc-gen: a protocol compiler for the XDL definition language
 * Copyright (C) 2006 Advanced Software Production Line, S.L.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 */
#include <xml-rpc-c-server.h>

/* write initial header comments */
void xml_rpc_c_server_write_header (gchar * comp_name)
{
	xml_rpc_support_multiple_write ("/**\n",
					" * C server skel to implement services exported by the XML-RPC\n",
					NULL);
	xml_rpc_support_write (" * component: %s.\n", comp_name);
	xml_rpc_support_multiple_write (" *\n",
					" * This file was generated by xml-rpc-gen tool, from Vortex Library\n",
					" * project. \n",
					" *\n",
					" * Vortex Library homepage:           http://vortex.aspl.es\n",
					" * Axl Library homepage:              http://xml.aspl.es\n",
					" * Advanced Software Production Line: http://www.aspl.es\n",
					" */\n", NULL);
	return;
}

/** 
 * @internal
 *
 * Writes all includes that enables the access to functions XML-RPC
 * exported.
 * 
 * @param doc The axlDoc that holds the interface.
 *
 * @param comp_name The component name.
 */
void xml_rpc_write_c_server_include_services (axlDoc * doc, 
					      gchar  * comp_name)
{
	axlNode * service;
	axlNode * name;
	axlNode * params;

	gchar   * content;
	
	/* get a reference to the first <service> node */
	service = axl_doc_get (doc, "/xml-rpc-interface/service");
	
	do {
		/* get the service name and write the include */
		name    = axl_node_get_child_nth (service, 0);
		content = axl_node_get_content_trim (name, NULL);

		/* write the initial header */
		xml_rpc_support_write ("#include <%s_%s", comp_name, content);

		/* get the <params> node */
		params = axl_node_get_child_nth (service, 2);
		if (NODE_CMP_NAME (params, "resource"))
			params = axl_node_get_next (params);

		/* write type prefixes */
		xml_rpc_support_write_function_type_prefix (params);
		
		/* write include trail */
		xml_rpc_support_write (".h>\n");

		/* get the next service */
	}while ((service = axl_node_get_next (service)) != NULL);

	/* write the last separator */
	xml_rpc_support_write ("\n\n");

	return;
}

/** 
 * @internal
 * 
 * Writes the default service dispatch function, that recognizes the
 * service and dispath it to the appropiate handler.
 *
 * @param doc The \ref axlDoc reference representing the interface.
 *
 * @param comp_name The XML-RPC server component name.
 */
void xml_rpc_write_c_server_default_service_dispath (axlDoc * doc, 
						     gchar  * comp_name)
{
	/* service location */
	axlNode * service;
	gchar   * service_name;

	axlNode * name;
	axlNode * params;
	axlNode * param;

	/* service type */
	axlNode * type;
	gchar   * service_type;
	
	/* write initial header */
	xml_rpc_support_write ("XmlRpcMethodResponse *  service_dispatch (VortexChannel * channel, XmlRpcMethodCall * method_call, gpointer user_data)\n{\n\n");

	/* push the indent */
	xml_rpc_support_push_indent ();
	
	xml_rpc_support_write ("/* check for a match for a service */\n");
	
	/* get a reference to the first service node. */
	service = axl_doc_get (doc, "/xml-rpc-interface/service");
	do {
		/* get the service name */
		name         = axl_node_get_child_nth (service, 0);
		service_name = axl_node_get_content_trim (name, NULL);

		/* get the params node */
		params = axl_node_get_child_nth (service, 2);
		if (NODE_CMP_NAME (params, "resource")) {
			/* it is defined the resource for this
			 * service, skip it*/
			params = axl_node_get_next (params);
		}

		/* write initial service dispath code */
		xml_rpc_support_write ("if (method_call_is (method_call, \"%s\", %d, ", 
				       service_name, axl_node_get_child_num (params));

		/* now write the service type especification */
		if (axl_node_have_childs (params)) {
			param = axl_node_get_child_nth (params, 0);
			do {
				/* get the type inside the param */
				type         = axl_node_get_child_nth (param, 1);
				service_type = axl_node_get_content_trim (type, NULL);
				
				if (axl_cmp (service_type, "int"))
					xml_rpc_support_sl_write ("XML_RPC_INT_VALUE, ");
				
				if (axl_cmp (service_type, "boolean"))
					xml_rpc_support_sl_write ("XML_RPC_BOOLEAN_VALUE, ");
				
				if (axl_cmp (service_type, "string"))
					xml_rpc_support_sl_write ("XML_RPC_STRING_VALUE, ");
				
				if (axl_cmp (service_type, "base64"))
					xml_rpc_support_sl_write ("XML_RPC_BASE64_VALUE, ");
				
				if (axl_cmp (service_type, "double"))
					xml_rpc_support_sl_write ("XML_RPC_DOUBLE_VALUE, ");
				
				if (axl_cmp (service_type, "struct"))
					xml_rpc_support_sl_write ("XML_RPC_STRUCT_VALUE, ");
				
				if (axl_cmp (service_type, "array"))
					xml_rpc_support_sl_write ("XML_RPC_ARRAY_VALUE, ");
				
				/* get the next param */
			}while ((param = axl_node_get_next (param)) != NULL);
		} /* if */

		/* write support ending */
		xml_rpc_support_sl_write (" -1))\n");

		/* push again the indent */
		xml_rpc_support_push_indent ();

		/* write service invocator */
		xml_rpc_support_write ("return __%s_%d", service_name, axl_node_get_child_num (params));

		/* write type prefixes */
		xml_rpc_support_write_function_type_prefix (params);

		/* write invocation termination */
		xml_rpc_support_sl_write (" (method_call);\n\n");

		/* pop the indent */
		xml_rpc_support_pop_indent ();

		/* all parameters written */
	}while ((service = axl_node_get_next (service)) != NULL);

	/* write the unrecognized method */
	xml_rpc_support_write ("/* return that the method to be invoked, is not supported */\n");
	xml_rpc_support_write ("return CREATE_FAULT_REPLY (-1, \"Method call received couldn't be dispatched because it not supported by this server\");\n");
	
	/* restore the indent before returning */
	xml_rpc_support_pop_indent ();

	xml_rpc_support_write ("}\n\n");
	
	return;
}


/** 
 * @internal
 *
 * Writes the main function, that starts the xml-rpc listener server.
 * 
 */
void xml_rpc_write_c_server_main_function ()
{
	/* write the main header */
	xml_rpc_support_write ("gint main (gint argc, gchar ** argv)\n{\n\n");
	
	/* push the current indent */
	xml_rpc_support_push_indent ();
	
	/* write several lines */
	xml_rpc_support_multiple_write ("/* init vortex library */\n",
					"vortex_init ();\n\n",
					"/* enable XML-RPC profile */\n",
					"vortex_xml_rpc_accept_negociation (\n",
					NULL);
	
	/* push again the indent */
	xml_rpc_support_push_indent ();
	
	xml_rpc_support_multiple_write ("/* no resource validation function */\n",
					"NULL,\n",
					"/* no user space data for the validation resource\n",
					"* function. */\n",
					"NULL,\n",
					"service_dispatch,\n",
					"/* no user space data for the dispatch function. */\n",
					"NULL);\n\n",
					NULL);

	/* restore the previous indent */
	xml_rpc_support_pop_indent ();

	xml_rpc_support_multiple_write ("/* parse configuration and start the listener */\n",
					"if (! vortex_xml_rpc_listener_parse_conf_and_start_listeners ())\n",
					NULL);

	/* push the indent for the return */
	xml_rpc_support_push_indent ();

	xml_rpc_support_write ("return -1;\n\n");

	/* pop de indent for the return */
	xml_rpc_support_pop_indent ();
					
	xml_rpc_support_multiple_write ("/* wait for listeners (until vortex_exit is called) */\n",
					"vortex_listener_wait ();\n\n",
					"/* end vortex function */\n",
					"vortex_exit ();\n\n",
					"return 0;\n",
					NULL);

	/* write the last brace */
	xml_rpc_support_pop_indent ();

	xml_rpc_support_write ("}\n");

	return;
}

/** 
 * @internal
 *
 * Creates the main.c file for the provided component name.
 * 
 * @param doc The xml document that represents the interface.
 *
 * @param out_dir The directory where the main.c file will be
 * generated.
 *
 * @param comp_name The component name that the XML-RPC have.
 */
void xml_rpc_c_server_create_main_c (axlDoc * doc, 
				     gchar  * out_dir, 
				     gchar  * comp_name)
{
	/* drop a log */
	xml_rpc_report ("generating server stub at: %s..\n", out_dir);

	/* open the main.c file */
	xml_rpc_support_open_file ("%s/main.c", out_dir);

	/* write initial header comments */
	xml_rpc_c_server_write_header (comp_name);

	/* write all header includes that correspond to services */
	xml_rpc_support_write ("#include <vortex.h>\n");
	xml_rpc_write_c_server_include_services (doc, comp_name);
	
	/* now write the default service dispath */
	xml_rpc_write_c_server_default_service_dispath (doc, comp_name);

	/* write main function */
	xml_rpc_write_c_server_main_function ();

	/* close the file */
	xml_rpc_support_close_file ();

	return;
}

/** 
 * @internal
 * 
 * Writes to the opened file the service header, the service that will
 * execute the service code to be invoked.
 */
void xml_rpc_c_server_write_service_header (gchar   * service_name, 
					    gint      param_count, 
					    gchar   * return_type, 
					    gchar   * type_prefix,
					    axlNode * aux,
					    gboolean  is_header)
{
	axlDoc * doc = axl_node_get_doc (aux);

	/* write service prefix */
	if (axl_cmp (return_type, "int"))
		xml_rpc_support_write ("int ");

	if (axl_cmp (return_type, "boolean"))
		xml_rpc_support_write ("int ");

	if (axl_cmp (return_type, "double"))
		xml_rpc_support_write ("double ");

	if (axl_cmp (return_type, "string"))
		xml_rpc_support_write ("char * ");

	if (axl_cmp (return_type, "base64"))
		xml_rpc_support_write ("char * ");

	if (xml_rpc_c_stub_type_is_array (doc, return_type) ||
	    xml_rpc_c_stub_type_is_struct (doc, return_type))
		xml_rpc_support_write ("%s * ", return_type);
	    
	/* write the service header implementation */
	xml_rpc_support_write ("%s_%d%s (", service_name, param_count, type_prefix);
	
	/* write service parameters */
	xml_rpc_support_write_function_parameters (doc, aux);
	
	if (axl_node_have_childs (aux))
		xml_rpc_support_sl_write (", ");
	
	xml_rpc_support_sl_write ("char ** fault_error, int * fault_code)");

	if (is_header)
		xml_rpc_support_sl_write (";");

	return;
}

/** 
 * @internal
 *
 * Declares all struct, array, string, base64 variables that will be
 * required while unmarshalling.
 * 
 * @param params The root node that contains the parameters.
 */
gboolean xml_rpc_c_server_write_temporal_parameter_getting (gchar * comp_name, axlNode * params)
{
	axlNode * aux;
	axlNode * aux2;
	axlNode * aux3;	
	axlDoc  * doc;

	gchar   * type;
	gchar   * type_lower;
	gchar   * comp_name_lower = NULL;
	gchar   * name;

	gint      iterator;
	gboolean  first_time = TRUE;

	/* now write the service parameter spec */
	iterator = 0;
	if (axl_node_have_childs (params)) {
		/* get the xml document container */
		doc             = axl_node_get_doc (params);
		comp_name_lower = axl_stream_to_lower_copy (comp_name);
		
		/* the params node have childs, which means the
		 * service have parameters */
		aux2 = axl_node_get_child_nth (params, 0);
		do {
			/* get a reference to the type node */
			aux3 = axl_node_get_child_nth (aux2, 1);
			
			/* get the type */
			type = axl_node_get_content_trim (aux3, NULL);

			/* get a reference to the name */
			aux  = axl_node_get_child_nth (aux2, 0);
			
			/* get the type */
			name = axl_node_get_content_trim (aux, NULL);

			/* check for first time */
			if (xml_rpc_c_stub_type_is_struct (doc, type) ||
			    xml_rpc_c_stub_type_is_array (doc, type)) {
				if (first_time) {
					xml_rpc_support_write ("/* temporal variable declaration */\n");
					xml_rpc_support_write ("gboolean unmarshall_failure = FALSE;\n");
					first_time = FALSE;
				}
			}

			
			/* write type found */
			if (xml_rpc_c_stub_type_is_struct (doc, type) ||
			    xml_rpc_c_stub_type_is_array (doc, type)) {
				/* get lower type name */
				type_lower  = axl_stream_to_lower_copy (type);

				xml_rpc_support_write ("%s * %s = %s_%s_unmarshall (method_call_get_param_value_as_struct (method_call, %d), FALSE);\n", 
						       type, name, comp_name_lower, type_lower, iterator);
				/* free type lower */
				axl_free (type_lower);
				
			}
			
			/* update the iterator count */
			iterator++;

		}while ((aux2 = axl_node_get_next (aux2)) != NULL);

		/* the params node have childs, which means the
		 * service have parameters */
		aux2 = axl_node_get_child_nth (params, 0);
		do {
			/* get a reference to the type node */
			aux3 = axl_node_get_child_nth (aux2, 1);
			
			/* get the type */
			type = axl_node_get_content_trim (aux3, NULL);

			/* get a reference to the name */
			aux  = axl_node_get_child_nth (aux2, 0);
			
			/* get the type */
			name = axl_node_get_content_trim (aux, NULL);

			/* write type found */
			if (xml_rpc_c_stub_type_is_struct (doc, type) ||
			    xml_rpc_c_stub_type_is_array (doc, type)) {
				xml_rpc_support_write ("if (%s == NULL)\n", name);
				xml_rpc_support_write ("\tunmarshall_failure = TRUE;\n");
			}
			
			/* update the iterator count */
			iterator++;

		}while ((aux2 = axl_node_get_next (aux2)) != NULL);
	} /* end if */

	/* check for a temporal variable declaration */
	if (! first_time) {
		xml_rpc_support_write ("\n");
	}

	/* free allocated comp name */
	if (comp_name_lower != NULL)
		axl_free (comp_name_lower);

	/* nothing more */
	return (!first_time);
}

/** 
 * @internal
 *
 * Writes all deallocation instructions to release temporal variables.
 * 
 * @param params The root node that contains the parameters.
 */
void xml_rpc_c_server_write_temporal_parameter_releasing (gchar * comp_name, 
							  axlNode * params)
{
	axlNode * aux;
	axlNode * aux2;
	axlNode * aux3;	
	axlDoc  * doc;

	gchar   * type;
	gchar   * type_lower;
	gchar   * comp_name_lower = NULL;
	gchar   * name;

	gint      iterator;
	gboolean  first_time = TRUE;


	/* now write the service parameter spec */
	iterator = 0;
	if (axl_node_have_childs (params)) {
		/* get the xml document container */
		doc             = axl_node_get_doc (params);
		comp_name_lower = axl_stream_to_lower_copy (comp_name);
		
		/* the params node have childs, which means the
		 * service have parameters */
		aux2 = axl_node_get_child_nth (params, 0);
		do {
			/* get a reference to the type node */
			aux3 = axl_node_get_child_nth (aux2, 1);
			
			/* get the type */
			type = axl_node_get_content_trim (aux3, NULL);

			/* get a reference to the name */
			aux  = axl_node_get_child_nth (aux2, 0);
			
			/* get the type */
			name = axl_node_get_content_trim (aux, NULL);

			/* check for first time */
			if (xml_rpc_c_stub_type_is_struct (doc, type) ||
			    xml_rpc_c_stub_type_is_array (doc, type)) {
				if (first_time) {
					xml_rpc_support_write ("/* temporal variable deallocation */\n");
					first_time = FALSE;
				}
			}

			
			/* write type found */
			if (xml_rpc_c_stub_type_is_struct (doc, type) ||
			    xml_rpc_c_stub_type_is_array (doc, type)) {
				/* get lower type name */
				type_lower  = axl_stream_to_lower_copy (type);
				
				xml_rpc_support_write ("%s_%s_free (%s);\n", 
						       comp_name_lower, type_lower, name);
				
				/* free type lower */
				axl_free (type_lower);
			}
			
			/* update the iterator count */
			iterator++;

		}while ((aux2 = axl_node_get_next (aux2)) != NULL);
	}

	/* check for a temporal variable declaration */
	if (! first_time) {
		xml_rpc_support_write ("\n");
	}

	/* free allocated comp name */
	if (comp_name_lower != NULL)
		axl_free (comp_name_lower);

	/* nothing more */
	return;
}

/** 
 * @internal
 * 
 * Writes the parameter getting sentences required to get from the
 * XmlRpcMethodCall, all values inside, to be passed in to the user
 * implementation.
 * 
 * @param params An axlNode pointing to the <params> node.
 */
gboolean xml_rpc_c_server_write_parameter_getting (axlNode * params)
{
	axlNode * aux;
	axlNode * aux2;
	axlNode * aux3;	
	axlDoc  * doc;

	gchar   * type;
	gchar   * name;

	gint      iterator;


	/* now write the service parameter spec */
	iterator = 0;
	if (axl_node_have_childs (params)) {

		/* get the xml document container */
		doc = axl_node_get_doc (params);
		
		/* the params node have childs, which means the
		 * service have parameters */
		aux2 = axl_node_get_child_nth (params, 0);
		do {
			/* get a reference to the type node */
			aux3 = axl_node_get_child_nth (aux2, 1);
			
			/* get the type */
			type = axl_node_get_content_trim (aux3, NULL);

			/* get a reference to the name node */
			aux  = axl_node_get_child_nth (aux2, 0);
			
			/* get the type */
			name = axl_node_get_content_trim (aux, NULL);
			
			/* write type found */
			if (xml_rpc_c_stub_type_is_struct (doc, type) ||
			    xml_rpc_c_stub_type_is_array (doc, type))
				xml_rpc_support_sl_write ("%s, ", name);
			else
				xml_rpc_support_sl_write ("method_call_get_param_value_as_%s (method_call, %d), ",
							  type, iterator);

			/* update the iterator count */
			iterator++;

		}while ((aux2 = axl_node_get_next (aux2)) != NULL);
	}

	/* nothing more */
	return;
}

/** 
 * @internal
 *
 * Write the service file, that implements the service unmarshaller
 * and the service implementation.
 * 
 * @param service The service node that contains all spec information.
 *
 * @param out_dir The directory where the service generation will be
 * placed.
 *
 * @param comp_name The component name, that is, the XML-RPC
 * component.
 */
void xml_rpc_c_server_create_write_service (axlNode * service, 
					    gchar   * out_dir, 
					    gchar   * comp_name)
{
	gchar   * service_name;
	gchar   * return_type;
	gchar   * return_type_lower;

	gint      param_count;

	axlNode * aux;
	axlDoc  * doc;

	axlNode * code;
	gchar   * service_content;

	gchar   * type_prefix;

	gchar   * comp_name_upper;
	gchar   * comp_name_lower;
	gchar   * service_name_upper;

	/* get service name */
	aux          = axl_node_get_child_nth (service, 0);
	service_name = axl_node_get_content_trim (aux, NULL);
	doc          = axl_node_get_doc (service);

	/* get upper values */
	comp_name_upper    = axl_stream_to_upper_copy (comp_name);
	comp_name_lower    = axl_stream_to_lower_copy (comp_name);
	service_name_upper = axl_stream_to_upper_copy (service_name);
	
	/* get return type */
	aux               = axl_node_get_next (aux);
	return_type       = axl_node_get_content_trim (aux, NULL);
	return_type_lower = axl_stream_to_lower_copy (return_type);

	/* check if the service is inside a resource or next node is
	 * the params */
	aux          = axl_node_get_next (aux);
	if (NODE_CMP_NAME (aux, "resource")) {
		/* get the reference to the params node */
		aux  = axl_node_get_next (aux);
	}
	
	/* open the service implementation file */
	type_prefix  = xml_rpc_support_get_function_type_prefix (aux);

	param_count  = axl_node_get_child_num (aux);

	/* open the file */
	xml_rpc_support_open_file ("%s/%s_%s%s.h", 
				   out_dir, 
				   comp_name_lower, 
				   service_name, 
				   (type_prefix != NULL) ? type_prefix : "");

	/* write initial header */
	xml_rpc_c_server_write_header (comp_name);

	xml_rpc_support_write ("#ifndef __SERVER_%s_%s_XML_RPC_H__\n",
			       comp_name_upper, service_name_upper);
	
	xml_rpc_support_write ("#define __SERVER_%s_%s_XML_RPC_H__\n\n",
			       comp_name_upper, service_name_upper);

	xml_rpc_support_write ("#include <vortex.h>\n");
	xml_rpc_support_write ("#include <%s_types.h>\n\n",
			       comp_name_lower);

	/* write the service header definition */
	xml_rpc_c_server_write_service_header (service_name, param_count, return_type, type_prefix,
					       aux, TRUE);
	xml_rpc_support_write ("\n\n");

	/* write the service unmarshaller definition */
	xml_rpc_support_write ("XmlRpcMethodResponse * __%s_%d%s (XmlRpcMethodCall * method_call);\n\n",
			       service_name, param_count, type_prefix);
	
	xml_rpc_support_write ("#endif\n");
	
	/* close the file */
	xml_rpc_support_close_file ();

	/* now write the service body implementation */
	xml_rpc_support_open_file ("%s/%s_%s%s.c", 
				   out_dir, 
				   comp_name_lower, 
				   service_name, 
				   (type_prefix != NULL) ? type_prefix : "");
	/* write initial header */
	xml_rpc_c_server_write_header (comp_name);

	xml_rpc_support_write ("#include <vortex.h>\n");
	xml_rpc_support_write ("#include <%s_types.h>\n\n",
			       comp_name_lower);

	/* write the user service, in the mean time, empty */
	xml_rpc_c_server_write_service_header (service_name, param_count, return_type, type_prefix,
					       aux, FALSE);
	xml_rpc_support_write ("\n{\n");
	
	/* push the indent to write the empty body */
	xml_rpc_support_push_indent ();

	/* check if the user have defined a service implementation, if
	 * not, put the WRITE HERE notice */

	/* get a reference to the optional code node */
	code = axl_node_get_next (aux);
	if (NODE_CMP_NAME (code, "code")) {
		/* get a reference to the user service code */
		code            = axl_node_get_child_nth (code, 0);
		service_content = axl_node_get_content (code, NULL);

		/* write the user code */
		xml_rpc_support_write (service_content);
	}else {
		xml_rpc_support_write ("/* WRITE HERE YOUR CODE */\n");
		xml_rpc_support_write ("REPLY_FAULT (\"Service is not implemented yet.\", -1, ");

		if (axl_cmp (return_type, "int") || 
		    axl_cmp (return_type, "double") || 
		    axl_cmp (return_type, "boolean"))
			xml_rpc_support_sl_write ("0");
	
		if (axl_cmp (return_type, "string") || 
		    axl_cmp (return_type, "struct") || 
		    axl_cmp (return_type, "array") || 
		    axl_cmp (return_type, "base64"))
			xml_rpc_support_sl_write ("NULL");
		
		xml_rpc_support_sl_write (");\n");
	}

	/* restore the indent */
	xml_rpc_support_pop_indent ();

	xml_rpc_support_write ("\n}\n\n");

	/* write the service unmarshaller definition */
	xml_rpc_support_write ("/* This is a support function to invoke \'%s\' service , do not modify it!! */\n",
			       service_name);
	xml_rpc_support_write ("XmlRpcMethodResponse * __%s_%d%s (XmlRpcMethodCall * method_call)\n{\n",
			       service_name, param_count, type_prefix);

	/* push the indent to write the service body */
	xml_rpc_support_push_indent ();

	/* write variable declarations */
	xml_rpc_support_multiple_write ("/* error support variables */\n",
					"char * fault_error = NULL;\n",
					"int    fault_code  = -1;\n",
					NULL);

	/* write the variable that will hold the value */
	if (axl_cmp (return_type, "int"))
		xml_rpc_support_write ("int ");

	if (axl_cmp (return_type, "double"))
		xml_rpc_support_write ("double ");

	if (axl_cmp (return_type, "boolean"))
		xml_rpc_support_write ("int ");

	if (axl_cmp (return_type, "string"))
		xml_rpc_support_write ("char * ");

	if (axl_cmp (return_type, "base64"))
		xml_rpc_support_write ("char * ");

	if (xml_rpc_c_stub_type_is_struct (doc, return_type) ||
	    xml_rpc_c_stub_type_is_array (doc, return_type)) {
		xml_rpc_support_write ("%s * ", return_type);
	}

	if (axl_cmp (return_type, "string") ||
	    axl_cmp (return_type, "base64") ||
	    xml_rpc_c_stub_type_is_struct (doc, return_type) ||
	    xml_rpc_c_stub_type_is_array (doc, return_type))
		xml_rpc_support_sl_write ("   result = NULL;\n\n");
	else
		xml_rpc_support_sl_write ("   result;\n\n");

	if (xml_rpc_c_stub_type_is_array (doc, return_type))
		xml_rpc_support_write ("XmlRpcArray * _result;\n");
	else if (xml_rpc_c_stub_type_is_struct (doc, return_type))
		xml_rpc_support_write ("XmlRpcStruct * _result;\n");

	/* check for variables to be declared and write them */
	if (xml_rpc_c_server_write_temporal_parameter_getting (comp_name, aux)) {
		/* write the invocation */
		xml_rpc_support_write ("/* call to the user implementation */\n");
		xml_rpc_support_write ("if (! unmarshall_failure)\n");
		xml_rpc_support_write ("\tresult = %s_%d%s (", service_name, param_count, type_prefix);
	} else {
		/* write the invocation */
		xml_rpc_support_write ("/* call to the user implementation */\n");
		xml_rpc_support_write ("result = %s_%d%s (", service_name, param_count, type_prefix);
	}

	/* write parameters */
	xml_rpc_c_server_write_parameter_getting (aux);

	xml_rpc_support_sl_write (" &fault_error, &fault_code);\n\n");

	/* check for temporal variables to be released */
	xml_rpc_c_server_write_temporal_parameter_releasing (comp_name, aux);

	/* write result checking for a negative reply */
	xml_rpc_support_write ("/* check error reply looking at the fault_error */\n");
	
	xml_rpc_support_write ("if (fault_error != NULL) {\n");

	xml_rpc_support_push_indent ();
	xml_rpc_support_multiple_write ("/* we have a error reply */\n",
					"return CREATE_FAULT_REPLY (fault_code, fault_error);\n",
					NULL);
	xml_rpc_support_pop_indent ();

	xml_rpc_support_write ("}\n\n");

	/* write additionall checking for null pointers */
	if (xml_rpc_c_stub_type_is_array (doc, return_type) ||
	    xml_rpc_c_stub_type_is_struct (doc, return_type) ||
	    axl_cmp (return_type, "string") || 
	    axl_cmp (return_type, "base64")) {
		xml_rpc_support_write ("if (result == NULL) {\n");
		
		xml_rpc_support_push_indent ();
		xml_rpc_support_multiple_write ("/* we have a error reply */\n",
						"return CREATE_FAULT_REPLY (-2, \"Returned a NULL value from a service which returns a pointer type. This is not allowed.\");\n",
						NULL);
		xml_rpc_support_pop_indent ();
		
		xml_rpc_support_write ("}\n\n");
	}


	/* write unmarshalling code for array and structs */
	if (xml_rpc_c_stub_type_is_array (doc, return_type) ||
	    xml_rpc_c_stub_type_is_struct (doc, return_type)) {
		xml_rpc_support_write ("/* Translate structure returned by the service */\n");
		xml_rpc_support_write ("_result = %s_%s_marshall (result, TRUE);\n\n",
				       comp_name_lower, return_type_lower);
	}

	xml_rpc_support_multiple_write ("/* return reply generated */\n",
					"return CREATE_OK_REPLY (", 
					NULL);

	/* write the variable that will hold the value */
	if (axl_cmp (return_type, "int"))
		xml_rpc_support_sl_write ("XML_RPC_INT_VALUE, GINT_TO_POINTER (result));\n");

	if (axl_cmp (return_type, "double"))
		xml_rpc_support_sl_write ("XML_RPC_DOUBLE_VALUE, &result);\n");

	if (axl_cmp (return_type, "boolean"))
		xml_rpc_support_sl_write ("XML_RPC_BOOLEAN_VALUE, GINT_TO_POINTER (result));\n");

	if (axl_cmp (return_type, "string"))
		xml_rpc_support_sl_write ("XML_RPC_STRING_REF_VALUE, result);\n");

	if (axl_cmp (return_type, "base64"))
		xml_rpc_support_sl_write ("XML_RPC_BASE64_REF_VALUE, result);\n");

	if (xml_rpc_c_stub_type_is_struct (doc, return_type))
		xml_rpc_support_sl_write ("XML_RPC_STRUCT_VALUE, _result);\n");

	if (xml_rpc_c_stub_type_is_array (doc, return_type))
		xml_rpc_support_sl_write ("XML_RPC_ARRAY_VALUE, _result);\n");
	

	/* restore the indent and close the function */
	xml_rpc_support_pop_indent ();
	xml_rpc_support_write ("}\n");

	/* close the file */
	xml_rpc_support_close_file ();

	/* release memory used by allocated variables */
	axl_free (comp_name_lower);
	axl_free (comp_name_upper);
	axl_free (service_name_upper);
	axl_free (return_type_lower);

	/* write the service unmarshaller */
	return;
}


/** 
 * @internal
 *
 * Write a module for every service exported.
 */
void xml_rpc_c_server_create_writes_services (axlDoc * doc, 
					      gchar  * result, 
					      gchar  * comp_name)
{
	axlNode * service;

	
	/* get the first service received */
	service = axl_doc_get (doc, "/xml-rpc-interface/service");
	do {
		/* write the service */
		xml_rpc_c_server_create_write_service (service, result, comp_name);
		
	}while ((service = axl_node_get_next (service)) != NULL);

	/* all services files writen */
	return;
}

/** 
 * @internal
 * 
 * Creates the default xml configuration for the server stub being
 * created. It contains default startup configuration such host and
 * port.
 */
void xml_rpc_c_server_create_default_xml_config (gchar * out_dir)
{
	/* open the xml configuration file */
	xml_rpc_support_open_file ("%s/conf.xml", out_dir);

	/* write default configuration */
	xml_rpc_support_multiple_write ("<vortex-listener>\n",
					"  <listener>\n",
					"     <hostname>0.0.0.0</hostname>\n",
					"     <port>44000</port>\n",
					"  </listener>\n",
					"</vortex-listener>\n", NULL);
	
	/* close the configuration file */
	xml_rpc_support_close_file ();
	
}

/** 
 * @internal
 *
 * Produces the server stub implementation, filling the services with
 * the optional code provided.
 * 
 * @param doc The \ref axlDoc reference where the xml document is
 * located.
 *
 * @param out_dir An output directory where the server stub will be
 * created.
 *
 * @param comp_name The XML-RPC component name.
 */
void xml_rpc_c_server_create (axlDoc * doc, gchar  * out_dir, 
			      gchar  * comp_name)
{
	gchar   * result;
	axlNode * service;

	/* create the the server stub directory */
	result = xml_rpc_support_create_dir ("%s/server-%s", out_dir, comp_name);

	/* create the main.c file */
	xml_rpc_c_server_create_main_c (doc, result, comp_name);

	/* write all services */
	xml_rpc_c_server_create_writes_services (doc, result, comp_name);

	/* write xml configuration file */
	xml_rpc_c_server_create_default_xml_config (result);

	/* write all struct and array definitions */
	xml_rpc_c_stub_write_all_struct_and_array_defs (doc, comp_name, result);

	/* write the type header */
	xml_rpc_c_stub_write_type_header (result, comp_name, doc);

	/* free the directory reference */
	g_free (result);
	
	return;
}
