/* Hey emacs, show me this like 'c': -*- c -*-
 *
 * xml-rpc-gen: a protocol compiler for the XDL definition language
 * Copyright (C) 2006 Advanced Software Production Line, S.L.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 */

#include <xml-rpc.h>

#define XML_RPC_GEN_CONSUME_SPACES(stream, error) \
axl_stream_consume_white_spaces (stream); xml_rpc_gen_translate_consume_comments (stream, error)

void xml_rpc_gen_translate_consume_comments (axlStream * stream, axlError ** error)
{
	bool found_item;
	
	/* know, try to read comments a process instructions.  Do this
	 * until both fails. Do this until one of them find
	 * something. */
	do {
		/* flag the loop to end, and only end if both,
		 * comments matching and PI matching fails. */
		found_item = AXL_FALSE;
		
		/* get rid from spaces */
		AXL_CONSUME_SPACES (stream);

		/* check for comments */
		if (axl_stream_inspect (stream, "/*", 2) > 0) {
			
			if (! axl_stream_get_until_ref (stream, NULL, NULL, AXL_TRUE, NULL, 1, "*/")) {
				axl_error_new (-1, "detected an opened comment but not found the comment ending",
					       stream, error);
				axl_stream_free (stream);
				return;
			} 
			
			/* flag that we have found a comment */
			found_item = AXL_TRUE;
		}

		/* get rid from spaces */
		AXL_CONSUME_SPACES(stream);

		if (axl_stream_inspect (stream, "//", 2) > 0) {
			if (! axl_stream_get_until_ref (stream, NULL, NULL, AXL_TRUE, NULL, 1, "\n")) {
				axl_error_new (-1, "detected an opened comment but not found the comment ending",
					       stream, error);
				axl_stream_free (stream);
				return;
			}

			/* flag that we have found a comment */
			found_item = AXL_TRUE;
		}
	
		/* get rid from spaces */
		AXL_CONSUME_SPACES(stream);

		/* check to break-the-loop */
	}while (found_item);

	return;
}

axlNode * __xml_rpc_gen_translate_struct (axlDoc * doc, bool * result, axlStream * stream, axlError ** error) 
{
	gchar   * name;
	gchar   * type;
	gint      chunk_matched;

	axlNode * _struct;
	axlNode * root;
	axlNode * node;
	axlNode * nodeAux;


	/* consume spaces and comments */
	XML_RPC_GEN_CONSUME_SPACES (stream, error);

	/* seems we have an struct definition, get the struct
	 * name */
	name = axl_stream_get_until (stream, NULL, &chunk_matched, AXL_TRUE, 1, " ");
	if (name == NULL) {
		(* result) = FALSE;
		axl_error_new (-2, "Expected to find the struct name but it wasn't found", 
			       stream, error);
		axl_stream_free (stream);
		return NULL;
	}
	
	/* create the struct node */
	_struct = axl_node_create ("struct");
	root    = axl_doc_get_root (doc);
	axl_node_set_child (root, _struct);
	
	/* add the struct name */
	node   = axl_node_create ("name");
	axl_stream_nullify (stream, LAST_CHUNK);
	axl_node_set_content_ref (node, name, -1);
	axl_node_set_child (_struct, node);
	
	/* now get the next open bracket */
	/* consume spaces */
	XML_RPC_GEN_CONSUME_SPACES(stream, error);
	
	/* now get the services types, first, the open parent */
	if (! (axl_stream_inspect (stream, "{", 1) > 0)) {
		(* result) = FALSE;
		axl_error_new (-2, "Expected to find an open bracket {, while reading a struct definition", 
			       stream, error);
		axl_stream_free (stream);
		return NULL;
	}
	
	/* parse all members */
	while (! (axl_stream_inspect (stream, "}", 1) > 0)) {
		/* consume spaces */
		XML_RPC_GEN_CONSUME_SPACES(stream, error);
		
		/* get the member type */
		type = axl_stream_get_until (stream, NULL, &chunk_matched, AXL_TRUE, 1, " ");
		axl_stream_nullify (stream, LAST_CHUNK);
		
		/* consume spaces */
		XML_RPC_GEN_CONSUME_SPACES(stream, error);
		
		/* get the member name */
		name = axl_stream_get_until (stream, NULL, &chunk_matched, AXL_TRUE, 2, " ", ";");
		axl_stream_nullify (stream, LAST_CHUNK);
		
		/* consume spaces */
		XML_RPC_GEN_CONSUME_SPACES(stream, error);
		
		if (chunk_matched == 0) {
			/* seems the user have placed a white
			 * space between the name and the
			 * semicolon */
			if (! (axl_stream_inspect (stream, ";", 1) > 0)) {
				(* result) = FALSE;
				axl_error_new (-2, "Expected to find a semi-colon (;), while reading a member name definition.", 
					       stream, error);
				axl_stream_free (stream);
				return NULL;
			}
		}
		
		/* add the member to the struct */
		node = axl_node_create ("member");
		axl_node_set_child (_struct, node);
		
		/* add the name to the member */
		nodeAux = axl_node_create ("name");
		axl_node_set_content_ref (nodeAux, name, -1);
		axl_node_set_child (node, nodeAux);
		
		/* add the type to the nameber  */
		nodeAux = axl_node_create ("type");
		axl_node_set_content_ref (nodeAux, type, -1);
		axl_node_set_child (node, nodeAux);
		
		/* consume spaces */
		XML_RPC_GEN_CONSUME_SPACES(stream, error);
	}
	
	/* consume spaces */
	XML_RPC_GEN_CONSUME_SPACES(stream, error);
	
	/* struct properly parsed */
	(* result) = TRUE;
	return _struct;
}

/** 
 * @internal Parse IDL definition file and translate it into an
 * in-memory XDL definition.
 */
axlNode * __xml_rpc_gen_translate_array (axlDoc * doc, bool * result, axlStream * stream, axlError ** error)
{
	axlNode * array;
	axlNode * root;
	axlNode * node;

	gchar   * array_name;
	gchar   * node_type;

	gint      chunk_matched;

	/* consume spaces and comments */
	XML_RPC_GEN_CONSUME_SPACES (stream, error);

	/* get the array name */
	array_name = axl_stream_get_until (stream, NULL, &chunk_matched, AXL_TRUE, 1, " ");
	axl_stream_nullify (stream, LAST_CHUNK);
	if (array_name == NULL) {
		(* result) = FALSE;
		axl_error_new (-2, "Expected to find the array name declaration but it wasn't found", 
			       stream, error);
		axl_stream_free (stream);
		return NULL;
	}

	/* consume spaces and comments */
	XML_RPC_GEN_CONSUME_SPACES (stream, error);

	/* get of keyword */
	if (! (axl_stream_inspect (stream, "of", 2) > 0)) {
		(* result) = FALSE;
		axl_error_new (-2, "Expected to find array \"of\" keyword, but it wasn't found. It separates the array declaration from the type holded inside it.", 
			       stream, error);
		axl_stream_free (stream);
		return NULL;
	}

	/* consume spaces and comments */
	XML_RPC_GEN_CONSUME_SPACES (stream, error);
	
	/* now get the node type inside the array */
	node_type = axl_stream_get_until (stream, NULL, &chunk_matched, AXL_TRUE, 2, " ", ";");
	axl_stream_nullify (stream, LAST_CHUNK);
	if (node_type == NULL) {
		(* result) = FALSE;
		axl_error_new (-2, "Expected to find the array node type declaration (the type inside the array) but it wasn't found", 
			       stream, error);
		axl_stream_free (stream);
		return NULL;
	}

	/* get last semicolon */
	if (chunk_matched == 0) {
		/* chunk matched was white space (in any form) */
		if (! (axl_stream_inspect (stream, ";", 1) > 0)) {
			(* result) = FALSE;
			axl_error_new (-2, "Expected to find a semicolon at the end of an array type declaration.", 
				       stream, error);
			axl_stream_free (stream);
			return NULL;
		}
	}

	/* now create the xml structure, representing the array */
	array = axl_node_create ("array");
	root  = axl_doc_get_root (doc);
	axl_node_set_child (root, array);
	
	/* add the array name */
	node   = axl_node_create ("name");
	axl_node_set_content_ref (node, array_name, -1);
	axl_node_set_child (array, node);

	/* add the array type */
	node   = axl_node_create ("type");
	axl_node_set_content_ref (node, node_type, -1);
	axl_node_set_child (array, node);

	/* add the size declaration (for now, always 0) */
	node   = axl_node_create ("size");
	axl_node_set_content (node, "0", 1);
	axl_node_set_child (array, node);
	
	/* return the array */
	(* result) = TRUE;
	return array;
}

axlNode * xml_rpc_gen_translate_parse_interface_item (axlDoc * doc, axlStream * stream, 
						      bool * result, axlError ** error)
{
	gchar   * return_type;
	gchar   * service_name;

	gchar   * type;
	gchar   * name;
	gchar   * user_code;
	gchar   * user_code_aux;
	gchar   * user_code_complete;

	axlNode * root;
	axlNode * service;
	axlNode * params = NULL;
	axlNode * code;
	axlNode * content;

	axlNode * _struct;
	axlNode * node;
	axlNode * nodeAux;

	gint      chunk_matched;
	gint      parents_opened;

	/* consume spaces */
	XML_RPC_GEN_CONSUME_SPACES (stream, error);

	/* get the service return type */
	return_type = axl_stream_get_until (stream, NULL, &chunk_matched, AXL_TRUE, 2, " ", "}");

	/* check that there is no more services defined */
	if (chunk_matched == 1) {
		(* result) = TRUE;
		return NULL;
	}

	/* nullify here to make return_type to be owned by this
	 * scope */
	axl_stream_nullify (stream, LAST_CHUNK);

	/* check for struct definition */
	if (axl_cmp (return_type, "struct")) {
		/* parse an struct */
		return __xml_rpc_gen_translate_struct (doc, result, stream, error);
	}

	if (axl_cmp (return_type, "array")) {
		/* parse an array */
		return __xml_rpc_gen_translate_array (doc, result, stream, error);
	}

	/* consume spaces */
	XML_RPC_GEN_CONSUME_SPACES(stream, error);

	/* now get the service name */
	service_name = axl_stream_get_until (stream, NULL, NULL, AXL_TRUE, 1, " ");
	axl_stream_nullify (stream, LAST_CHUNK);

	/* create the service node */
	service = axl_node_create ("service");
	root    = axl_doc_get_root (doc);

	axl_node_set_child (root, service);

	/* configure service name */
	node    = axl_node_create ("name");
	axl_node_set_content_ref (node, service_name, -1);
	axl_node_set_child (service, node);

	/* configure service return type */
	node    = axl_node_create ("returns");
	axl_node_set_content_ref (node, return_type, -1);
	axl_node_set_child (service, node);

	/* consume spaces */
	XML_RPC_GEN_CONSUME_SPACES(stream, error);

	/* now get the services types, first, the open parent */
	if (! (axl_stream_inspect (stream, "(", 1) > 0)) {
		(* result) = FALSE;
		axl_error_new (-2, "Expected to find an open parent (, while reading service definition", 
			      stream, error);
		axl_stream_free (stream);
		return NULL;
	}

	/* create the params node and add it to the service */
	params = axl_node_create ("params");
	axl_node_set_child (service, params);

	/* get service parameter definition */
	while (TRUE) {
		/* consume spaces */
		XML_RPC_GEN_CONSUME_SPACES(stream, error);

		/* check if the service parameter definition have
		 * ended */
		if (axl_stream_inspect (stream, ")", 1) > 0)
			break;

		/* get the parameter type */
		type = axl_stream_get_until (stream, NULL, NULL, AXL_TRUE, 1, " ");
		axl_stream_nullify (stream, LAST_CHUNK);
		
		/* consume spaces */
		XML_RPC_GEN_CONSUME_SPACES(stream, error);

		/* get the parameter name */
		name = axl_stream_get_until (stream, NULL, &chunk_matched, AXL_TRUE, 3, " ", ",", ")");
		axl_stream_nullify (stream, LAST_CHUNK);

		/* create the param node */
		node = axl_node_create ("param");
		axl_node_set_child (params, node);
		
		/* create the param name */
		nodeAux = axl_node_create ("name");
		axl_node_set_content_ref (nodeAux, name, -1);
		axl_node_set_child (node, nodeAux);
		
		/* create the param type */
		nodeAux = axl_node_create ("type");
		axl_node_set_content_ref (nodeAux, type, -1);
		axl_node_set_child (node, nodeAux);

		/* check matched chunk */
		if (chunk_matched == 2) {
			/* there are no more parameters */
			break;
		}

		if (chunk_matched == 0) {
			/* we need to consume the comma */

			/* consume spaces */
			XML_RPC_GEN_CONSUME_SPACES(stream, error);

			/* check for comma, and, if found, consume
			 * it */
			if (axl_stream_peek (stream, ",", 1) > 0)
				axl_stream_accept (stream);
		} /* end if */
	} /* end while */

	/* consume spaces */
	XML_RPC_GEN_CONSUME_SPACES(stream, error);

	/* now try to parse the service content if defined */
	if (axl_stream_inspect (stream, ";", 1) > 0) {
		/* the service doesn't have user code defined */
		(* result) = TRUE;
		return service;
	}

	/* if reached this place, it means that the user have defined
	 * usr code for the service */
	if (axl_stream_inspect (stream, "{", 1) > 0) {
		/* parse the service code definition */
		parents_opened = 0;

		/* configure node relations */
		code           = axl_node_create ("code");
		content        = axl_node_create ("content");

		/* configure node relations */
		axl_node_set_child (service, code);
		axl_node_set_child (code, content);

		while (TRUE) {

			/* get the user code for the service */
			user_code = axl_stream_get_until (stream, NULL, &chunk_matched, 
							  AXL_TRUE, 3, "};", "}", "{");
			axl_stream_nullify (stream, LAST_CHUNK);

			/* store service content */
			/* get current content node */
			user_code_aux      = axl_node_get_content (content, NULL);
			if (parents_opened == 0 && (chunk_matched == 0 || chunk_matched == 1)) {
				user_code_complete = axl_stream_strdup_printf ("%s%s",
									       (user_code_aux != NULL) ? user_code_aux : "",
									       user_code);
			} else {
				user_code_complete = axl_stream_strdup_printf ("%s%s%s",
									       (user_code_aux != NULL) ? user_code_aux : "",
									       user_code,
									       (chunk_matched == 2) ? "{" : "}");
			}
			
			/* set new user code */
			axl_node_set_content_ref (content, user_code_complete, -1);
			
			/* release prevous content */
			axl_free (user_code);

			/* seems that a closing parent have been
			 * received */
			if (chunk_matched == 0 || 
			    chunk_matched == 1) {
				if (parents_opened == 0)
					break;

				/* decreate current parent opened */
				parents_opened--;
			}

			/* detected an open parent, store it to
			 * properly detect service code */
			if (chunk_matched == 2) {
				parents_opened++;
			}
		}
	}

	(* result) = TRUE;
	return service;

}

bool xml_rpc_gen_translate_parse_services (axlDoc     * doc, 
					   axlStream  * stream, 
					   axlError  ** error)
{
	axlNode * service;
	axlNode * root;
	bool      result;

	/* get the root node */
	root = axl_doc_get_root (doc);

	/* parse the service */
	while (TRUE) {
		/* parse one service */
		service = xml_rpc_gen_translate_parse_interface_item (doc, stream, &result, error);

		/* check parse result */
		if (! result) 
			return FALSE;

		/* check if there are no more services to parse */
		if (service == NULL) 
			break;
	}

	return TRUE;
}

axlDoc * xml_rpc_gen_translate_idl_to_xdl (gchar * selected, axlError ** error)
{
	axlStream * stream;
	axlDoc    * doc;
	axlNode   * node;
	axlNode   * nodeAux;

	gchar     * string_aux;
	

	/* create the stream and try to parse */
	stream     = axl_stream_new (NULL, -1, selected, -1, error);
	string_aux = NULL;
	g_return_val_if_fail (stream, NULL);

	/* consume spaces */
	XML_RPC_GEN_CONSUME_SPACES(stream, error);

	/* we have already read the stream, now parse the document
	 * header */
	if (! axl_stream_inspect (stream, "xml-rpc", 7) > 0) {
		axl_error_new (-2, "Expected to find xml-rpc keyword at the begin of the interface definition", 
			      stream, error);
		axl_stream_free (stream);
		return NULL;
	}

	xml_rpc_report ("detected IDL format definition..\n");

	/* consume spaces */
	XML_RPC_GEN_CONSUME_SPACES(stream, error);

	/* we have already read the stream, now parse the document
	 * header */
	if (! axl_stream_inspect (stream, "interface", 9) > 0) {
		axl_error_new (-2, "Expected to find interface keyword at the begin of the interface definition", 
			      stream, error);
		axl_stream_free (stream);
		return NULL;
	}

	/* consume spaces */
	XML_RPC_GEN_CONSUME_SPACES(stream, error);

	/* now get the name of the interface */
	string_aux = axl_stream_get_until (stream, NULL, NULL, AXL_TRUE, 1, " ");
	axl_stream_nullify (stream, LAST_CHUNK);
	if (string_aux == NULL) {
		axl_error_new (-2, "Expected to find the interface name", stream, error);
		axl_stream_free (stream);
		return NULL;
	}

	xml_rpc_report ("detected xml-rpc definition: '%s'..\n", string_aux);
	
	/* create the document */
	doc  = axl_doc_create ("1.0", NULL, TRUE);
	node = axl_node_create ("xml-rpc-interface");
	axl_doc_set_root (doc, node);
	
	/* set the xml-rpc interface name */
	nodeAux = axl_node_create ("name");
	axl_node_set_content_ref (nodeAux, string_aux, -1);
	axl_node_set_child (node, nodeAux);

	/* link the document to the stream */
	axl_stream_link (stream, doc, (axlDestroyFunc) axl_doc_free);

	/* consume spaces */
	XML_RPC_GEN_CONSUME_SPACES(stream, error);
	
	/* get the open brace */
	if (! axl_stream_inspect (stream, "{", 1) > 0) {
		axl_error_new (-2, "Expected to find an open brace, enclosing services definition", stream, error);
		axl_stream_free (stream);
		return NULL;	
	}

	/* parse services: if something goes wrong, just return NULL
	 * because the function already deallocates the document and
	 * the stream */
	if (! xml_rpc_gen_translate_parse_services (doc, stream, error))
		return NULL;
	
	/* consume spaces */
	XML_RPC_GEN_CONSUME_SPACES(stream, error);
	
	/* get the open brace */
	if (! axl_stream_inspect (stream, "}", 1) > 0) {
		axl_error_new (-2, "Expected to find a close brace, closing services definition", stream, error);
		axl_stream_free (stream);
		return NULL;	
	}
	
	/* unlink all references */
	axl_stream_unlink (stream);
	axl_stream_free (stream);

	/* return the document */
	return doc;
}
